var express = require('express'),
    http = require('http'),
    path = require('path'),
    mongo = require('mongodb'),
    mongoose = require('mongoose'),
    passport = require('passport'), 
    FacebookStrategy = require('passport-facebook').Strategy,
    flash = require("connect-flash"),
    ObjectID = require("./node_modules/mongoose/node_modules/mongodb").ObjectID,
    FB = require('fb'),
    d3 = require('d3'),
    Person,
    Link,
    FbUsers;



var app = new express();




var mongodbURL = process.env.MONGOHQ_URL || "mongodb://localhost/hookedup";
mongoose.connect(mongodbURL);
var db = mongoose.connection;
db.on('error', console.error.bind(console, 'connection error:'));
db.once('open', function callback () {

  // People
  var personSchema = mongoose.Schema({
    name: 'string',
    fbId: 'string',
    author: 'string'
  });
  Person = mongoose.model('Person', personSchema);

  // Links
  var linkSchema = mongoose.Schema({
      source: 'ObjectId',
      target: 'ObjectId',
      value: 'Number',
      author: 'string'
  });
  Link = mongoose.model('Link', linkSchema);

  // FB Users
  var FacebookUserSchema = new mongoose.Schema({
    fbId: String,
    email: { type : String , lowercase : true},
    name: String,
    fbToken: String
  });
  Users = mongoose.model('fbs',FacebookUserSchema);

    // FB Friends
  var FacebookFriendSchema = new mongoose.Schema({
    fbId: String,
    name: String,
    author: 'string'
  });
  Friend = mongoose.model('Friend',FacebookFriendSchema);

});

// Auth config
var sessionSecret = process.env.SESSION_SECRET || "SHHHH ITS A SECRET";
var FACEBOOK_APP_ID = process.env.FACEBOOK_APP_ID || YOUR_FACEBOOK_APP_ID;
var FACEBOOK_APP_SECRET = process.env.FACEBOOK_SECRET || YOUR_FACEBOOK_APP_SECRET;
var FACEBOOK_CALLBACK_URL = process.env.FACEBOOK_CALLBACK_URL || "http://hookedup.herokuapp.com/auth/facebook/callback";


passport.use(new FacebookStrategy({
    clientID: FACEBOOK_APP_ID,
    clientSecret: FACEBOOK_APP_SECRET,
    callbackURL: FACEBOOK_CALLBACK_URL
  },
  function(accessToken, refreshToken, profile, done) {
    var theFbToken = accessToken;
    // console.log("token: " + accessToken);
    Users.findOne({fbId : profile.id}, function(err, oldUser){
        // console.log("token2: " + accessToken);
        if(oldUser){
            oldUser.fbToken = accessToken;
            done(null,oldUser);
        }else{
            // console.log(accessToken);
            var newUser = new Users({
                fbId : profile.id ,
                email : profile.emails[0].value,
                name : profile.displayName,
                fbToken: accessToken
            }).save(function(err,newUser){
                if(err) throw err;
                done(null, newUser);
            });
        }
    });
  }
));

passport.serializeUser(function(user, done) {
    done(null, user.id);
});


passport.deserializeUser(function(id, done) {
    Users.findById(id,function(err,user){
        if(err) done(err);
        if(user){
            done(null,user);
        }else{
            LocalUsers.findById(id, function(err,user){
                if(err) done(err);
                done(null,user);
            });
        }
    });
});


app.set('views', __dirname + '/views');
app.set('view engine', 'jade');
app.use(express.favicon());
app.use(express.cookieParser());
app.use(express.bodyParser());
app.use(express.session({ secret: sessionSecret }));
app.use(passport.initialize());
app.use(passport.session());
app.use(express.methodOverride());
app.use(flash());
app.use(app.router);
app.use(express.static(path.join(__dirname, 'public')));




/*
* Error Handling
*/
app.use(function(req, res, next){
  res.status(404);
  if (req.accepts('html')) {
    res.render('404', { url: req.url });
    return;
  }
  if (req.accepts('json')) {
    res.send({ error: 'Not found' });
    return;
  }
  res.type('txt').send('Not found');
});

app.use(function(err, req, res, next){
  res.status(err.status || 500);
  res.render('500', { error: err });
});




function authenticatedOrNot(req, res, next){
    if(req.isAuthenticated()){
        next();
    }else{
        res.redirect("/login");
    }
}

// function userExist(req, res, next) {
//     Users.count({
//         username: req.body.username
//     }, function (err, count) {
//         if (count === 0) {
//             next();
//         } else {
//             // req.session.error = "User Exist"
//             res.redirect("/singup");
//         }
//     });
// }

app.get("/auth/facebook", passport.authenticate("facebook",{ scope : "email"}));

app.get("/auth/facebook/callback",
    passport.authenticate("facebook",{ failureRedirect: '/login'}),
    function(req,res){
        res.redirect('/');

    }
);


app.get("/login", function(req, res){ 
    res.render("login");
});

app.get('/logout', function(req, res){
  req.logout();
  res.redirect('/');
});

app.get("/", function(req, res){ 
    if(req.isAuthenticated()){
      var graph = {};
      Person.find({author: req.user.id }, function (personerr, nodedocs) {
        graph.nodes = nodedocs;
        Link.find({author: req.user.id}, function (linkerr, linkdocs) {
          graph.links = linkdocs.map(function (thelink) { 
            return { source: graph.nodes.map(function(e) { return e._id.toString(); }).indexOf(thelink.source.toString()), 
                     target: graph.nodes.map(function(e) { return e._id.toString(); }).indexOf(thelink.target.toString()),
                     value: thelink.value 
                   }
          });
          // res.json(graph);
          res.render("graph", { graph: graph}); 
        });
      });
    }
    else {
      res.send('<h1>HookedUp</h1><a href="/auth/facebook">Login with Facebook</a> ');
    }
       
});

app.get("/graphdata", authenticatedOrNot, function(req, res){
  var graph = {};
  Person.find({author: req.user.id }, function (personerr, nodedocs) {
    graph.nodes = nodedocs;
    Link.find({author: req.user.id}, function (linkerr, linkdocs) {
      graph.links = linkdocs.map(function (thelink) { 
        return { source: graph.nodes.map(function(e) { return e._id.toString(); }).indexOf(thelink.source.toString()), 
                 target: graph.nodes.map(function(e) { return e._id.toString(); }).indexOf(thelink.target.toString()),
                 value: thelink.value 
               }
      });
      // res.json(graph);
      res.json(graph); 
    });
  });
});

app.get("/profile", authenticatedOrNot, function(req, res){ 
    res.render("profile", { user : req.user});
});

app.get('/people', authenticatedOrNot, function(req,res){
  Person.find({author: req.user.id}, function (err, docs) {
      res.json(docs);
    });
});

app.get('/users', authenticatedOrNot, function(req,res){
  Users.find({}, function (err, docs) {
      res.json(docs);
    });
});

app.get('/links', authenticatedOrNot, function(req,res){
  Link.find({author: req.user.id}, function (err, docs) {
      res.json(docs);
    });
});

app.get('/friends', authenticatedOrNot, function(req,res){
  Friend.find({}, function (err, docs) {
      res.json(docs);
    });
});

app.get('/getfriends', authenticatedOrNot, function(req,res){
  FB.api('/me/friends', {access_token: req.user.fbToken}, function (resp) {
    if(!resp || resp.error) {
      console.log(!resp ? 'error occurred' : resp.error);
      return;
    }
      Friend.remove({ author: req.user.fbId });
      resp.data.forEach(function(fbf) {
        console.log(fbf.name);
        var newFriend = new Friend({ fbId: fbf.id, name: fbf.name, author: req.user.id });
        newFriend.save();
      });
      res.send("added friends");
  });
});


app.post('/people',function(req,res){
  var person = new Person({ name: req.body.name, author: req.user.id });
  person.save(function (err) {
    if (err) { 
      console.log('uh oh');
     }
   else {
      res.send('user ' + req.user.id + 'added ' + req.body.name);
    }
  });
});

app.post('/links',function(req,res){
  var link = new Link({ source: req.body.source, target: req.body.target, value: req.body.value, author: req.user.id });
  link.save(function (err) {
    if (err) { 
      console.log('uh oh');
     }
   else {
      res.send('added new link ' + req.body.source + " " + req.body.target);
    }
  });
});


app.delete('/people/:id',function(req,res){
  Person.remove({ _id: req.params.id }, function (err) {
    if (err) {
      console.log('uh oh');
    }
    else {
      res.send('removed ' + req.params.id);
    }
  });
});

app.delete('/links/:id',function(req,res){
  Link.remove({ _id: req.params.id }, function (err) {
    if (err) {
      console.log('uh oh');
    }
    else {
      res.send('removed ' + req.params.id);
    }
  });
});


var port = process.env.PORT || 5000;
app.listen(port, function() {
  console.log("Listening on " + port);
});
