var mongo = require('mongodb');
var express = require('express');
var mongoose = require('mongoose');

var app = new express();

var Person,
    Link;

mongoose.connect('mongodb://localhost/node-graph');
var db = mongoose.connection;
db.on('error', console.error.bind(console, 'connection error:'));
db.once('open', function callback () {

  // People
  var personSchema = mongoose.Schema({
    name: 'string',
  });
  Person = mongoose.model('Person', personSchema);

  // Links
  var linkSchema = mongoose.Schema({
      source: 'ObjectId',
      target: 'ObjectId',
      value: 'Number',
  });
  Link = mongoose.model('Link', linkSchema);


});


app.use(express.static(__dirname + '/public'));
app.use(express.bodyParser());

app.get('/graph',function(req,res){
  var graph = {};
  Person.find({}, function (personerr, nodedocs) {
    graph.nodes = nodedocs;
    Link.find({}, function (linkerr, linkdocs) {
      graph.links = linkdocs.map(function (thelink) { 
        return { source: graph.nodes.map(function(e) { return e._id.toString(); }).indexOf(thelink.source.toString()), 
                 target: graph.nodes.map(function(e) { return e._id.toString(); }).indexOf(thelink.target.toString()),
                 value: thelink.value 
               }
      });
      res.json(graph);
    });
  });
});

app.get('/people',function(req,res){
  Person.find({}, function (err, docs) {
      res.json(docs);
    });
});

app.get('/links',function(req,res){
  Link.find({}, function (err, docs) {
      res.json(docs);
    });
});


app.post('/people',function(req,res){
  var person = new Person({ name: req.body.name });
  person.save(function (err) {
    if (err) { 
      console.log('uh oh');
     }
   else {
      res.send('added ' + req.body.name);
    }
  });
});

app.post('/links',function(req,res){
  var link = new Link({ source: req.body.source, target: req.body.target, value: req.body.value });
  link.save(function (err) {
    if (err) { 
      console.log('uh oh');
     }
   else {
      res.send('added new link ' + req.body.source + " " + req.body.target);
    }
  });
});


app.delete('/people/:id',function(req,res){
  Person.remove({ _id: req.params.id }, function (err) {
    if (err) {
      console.log('uh oh');
    }
    else {
      res.send('removed ' + req.params.id);
    }
  });
});

app.delete('/links/:id',function(req,res){
  Link.remove({ _id: req.params.id }, function (err) {
    if (err) {
      console.log('uh oh');
    }
    else {
      res.send('removed ' + req.params.id);
    }
  });
});


var port = process.env.PORT || 5000;
app.listen(port, function() {
  console.log("Listening on " + port);
});
